"""Basic setup for behave and chromedriver."""
import behave_webdriver
import os
import re
import time
from behave_webdriver.driver import ChromeOptions


def save_screenshot(context, step):
    """Save a screenshot to ./screenshots."""
    timestamp = time.strftime("%Y-%m-%dT%H:%M:%S")
    filename = re.sub('\W', '-', '{} failed {}'.format(timestamp,
                                                       str(step.name)))
    filepath = os.path.join('screenshots', filename + '.png')
    if not os.path.exists('screenshots'):
        os.mkdir('screenshots')

    print('Saving screenshot to %s' % filepath)
    context.behave_driver.save_screenshot(filepath)


def before_all(context):
    """Run at the very beginning."""
    userdata = context.config.userdata

    headless = userdata.get('headless', 'False')

    chrome_options = ChromeOptions()
    if headless == 'True':
        chrome_options.add_argument('--headless')
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument('--disable-dev-shm-usage')
    context.behave_driver = behave_webdriver.Chrome(
        chrome_options=chrome_options)


def before_tag(context, tag):
    def get_values(app):
        values = {}
        values['url'] = userdata.get(app + '.url')
        values['username'] = \
            userdata.get(app + '.username', 'admin')
        values['password'] = \
            userdata.get(app + '.password')
        assert values['url'], '{0}.url variable missing in' \
            'userdata. Provide it with "-D {0}.url".'.format(app)
        assert values['username'], '{0}.username variable missing in' \
            'userdata. Provide it with "-D {0}.username".'.format(app)
        assert values['password'], '{0}.password variable missing in' \
            'userdata. Provide it with "-D {0}.password".'.format(app)
        return(values)

    userdata = context.config.userdata
    if tag == 'grafana':
        context.grafana = get_values('grafana')

    if tag == 'nextcloud':
        context.nextcloud = get_values('nextcloud')



def after_all(context):
    """Cleanup after tests run."""
    context.behave_driver.quit()


def after_step(context, step):
    """Save screeshot if step fails."""
    if step.status == 'failed':
        save_screenshot(context, step)
        print("Console log output:")
        for message in context.behave_driver.get_log('browser'):
            print("  Level: {}".format(message['level']))
            print("  Timestamp: {}".format(message['timestamp']))
            print("  Source: {}".format(message['source']))
            print("  Message: {}\n".format(message['message']))

